function [table, dat]=CornerDetect_nodep(data, iter, bkw)
% This function will return the x,y location of corners
% It detects corners by appoximating sections of the data set as lines
% Only /good/ approximations are considered and those not close enough
% to each other or meeting other requirements are disregarded.

%   data - 2d scatter data
%   bkw  - clustering iterations
%       CornerDetect.m  v1.2
%       Alexander T. Hypes  11/2013

    %% Options
    r2min =     .8; % minimum r^2 to be considered a /good/ approximation
    maxdist =  250; % max eculidian distance between endpoints (for intersections)
    minlen =    16; % minimum continous line length to be considered 'good'
    minang =    45; % min angle at intersection
    maxang =   135; % max angle at intersection
    incr_by = .025; % r^2 must increase by this to continue merging a section
    nochg =  .0125; % r2 val must change by more nochg to be considered /different/ during merge
    dup_dist = 125; % merge duplicate corners closer than /dup_dist/
    lookahead =  6; % lookahead distance (# of bins) when detecting corners %warning this cat get murderous%
       
    table=[];
    if bkw; spc = 2; end
    for ii = 1:spc:iter
        %% Cluster & Merge
        if mod(ii,2)==1 % only cluster on odd runs
            dat = cluster_2(data); % cluster data into groups
        elseif mod(ii,2)==0
            dat=flipud(dat); % on even runs, simply flip the data and renumber
        end
        if ~isempty(dat)
            dat(:,3)= fwdmerge(dat, incr_by, nochg, maxdist); % perform a forward merge on the groups

            %% Characterization
            pdata = zeros(max(dat(:,3)), 5); % perform a linear regeression on each group
            % pdata = [ slope, y_int, r2val, bgn_idx, end_idx ]
            for i =1:max(dat(:,3))
               idx = find( dat(:,3) == i); % indicies of current cluster
               pdata(i,4:5) = [idx(1) idx(end)];
               [pdata(i,1), pdata(i,2), pdata(i,3)]=linreg(dat(idx,1), dat(idx,2));
            end
            tab=lsscrub( pdata, minlen); % get scrubbed table of slope values

            %% Corner Check
            if ~isempty(tab) % dont try to find corners in an empty list
                ctbl=cdetect(tab, dat, maxdist, minang, maxang, r2min, lookahead); % get x y locations of corners
                if ~isempty(ctbl) % dont save an empty lists
                    table=[table; ctbl]; % return the table
                end
            end
        else
            table=[];
        end
    end
    %% Duplicate Merge/Removal
    if ~isempty(table)
        table=dup_rem2(table,dup_dist);
    end
end

% performs a weighted merge of duplicates from a 2d vector if they closer than dup_dist in
% euclidian distance (weights are based on r^2 value)
function table=dup_rem2(table,dup_dist)
    if length(table) > 1 % sanity check
        d_mat = pdist2_sq(table(:,1:2)); % compute distance between every element;
        [m,n]=size(d_mat);
        i=2;
        j=1;
        wd=0; % watchdog counter
        while ( i<=m ) || ( j<=n )
            if (i~=j) && (d_mat(i,j) < dup_dist) % if they're close enough
                div = [table(i,3); table(j,3)] / ( table(i,3) + table(j,3) ); % perform a weighted average
                table(i,1) = [table(i,1), table(j,1)] * div;
                table(i,2) = [table(i,2), table(j,2)] * div;
                table(i,3) = ( table(i,3) + table(j,3) ) / 2;
                table(i,:) = [0, 0, 0];

                marks = find(table(:,3)); % remove zeros
                table=table(marks,:);
                d_mat = pdist2_sq(table(:,1:2)); % recompute distances
                [m,n]=size(d_mat); % rehash
                j=0;
                i=2;
            end
            j=j+1;
            if j>n
                i=i+1;
                j=1;
            end
            if i>m
                break
            end
            wd=wd+1;
            if wd>200 % watchdog counter until fully tested
                disp('CD: Duplicate Removal: Watchdog Limit Exceeded') % you know...just in case
                break;
            end
        end
    end
end

% segments listin [n x 1] into sections
% find discontinuities and split sections by chp_cz
function [listout] = cfp_chop2(listin, cutoff)
    chp_sz=15;
    if isempty(listin)
        listout=[]; % dont process empty lists
    else
        idx = [find(listin > cutoff); length(listin)]; % get list of starting points (minus position 1)
        if ~isempty(idx) % only process lists with elements
            listout = zeros(length(listin), 1);
            b=1; % starting point
            wpos=1;
            for i=1:length(idx) % for each break point found above
                dist = idx(i) - b; % get the section length
                if dist>2*chp_sz % if the section is big enough to split
                    for j=1:(floor(dist/chp_sz)-1) % split by 10s adding the rem to the last bin
                        jj = b+chp_sz-1; % get block end
                        listout(b:jj)=wpos;
                        wpos=wpos+1;
                        b=jj+1;
                    end
                    listout(b:idx(i))=wpos-1; % write remaining blocks
                else
                    listout(b:idx(i)) = wpos;
                    wpos=wpos+1;
                end
                b=idx(i)+1;
            end
        else
            listout=[];
        end
    end
end

% cluster data in bins based on mahalanobis distance (sort-of)
function [dat] = cluster_2(data)
    marks = find(data(:,2)); % get non-zero indicies
    dat = zeros(length(marks),3); % TODO: vectorize this with a static length for speed
    dat(:,1:2) = [data(marks,:)]; % remove y=0 elements
    
    idx = mdist(dat(:,1:2),1,2); % get next-hop distance table
    mdx = cfp_chop2(idx, 150);
    if isempty(idx)
        dat(:,3) = 0; % return all zeros if blank
    else
        dat(:,3) = mdx(:,1); % otherwise return the bin numbers
    end    
end

% finds intersections of line approximations
function [vect] = cdetect(tbl, dat, maxdist, minang, maxang, r2min, lookahead)
    [m, ~]=size(tbl); % get size of table
    if m<2; vect=[]; return; end; % if theres only one line, dont do anything
    ptable=zeros((m-1), 3); % init table
    wpos=1; % inital write position
    for i=2:m % for each adjacent pair in the data
        for j=1:lookahead
            if (i-1+j)>m; break; end
            s = ( ( dat(tbl(i-1,5),1) - dat(tbl(i-1+j,4),1) ).^2 + ( dat(tbl(i-1,5),2) - dat(tbl(i-1+j,4),2) ).^2 ).^.5; % pdist manual replacement calc
            if s < maxdist && (tbl(i-1+j,3)+tbl(i-1,3))> (2* r2min) % they close enough? are they good enough lines?
                theta=atand((tbl(i-1+j,1)-tbl(i-1,1))/(1+tbl(i-1+j,1)*tbl(i-1,1))); % find the angle at the intersection of the lines
                if theta>minang && theta<maxang % is it sharp enough to be a corner?
                    x=(tbl(i-1,2)-tbl(i-1+j,2))/(tbl(i-1+j,1)-tbl(i-1,1)); % find x loc of intersection
                    y=(tbl(i-1+j,1)*x+tbl(i-1+j,2)); % find y value at intersection
                    wgt = (tbl(i-1+j,3)+(tbl(i-1+j,3)))/2; % r^2 weight
                    ptable(wpos,:)=[x y wgt]; % add to table
                    wpos=wpos+1; % increment write position
                end
            end
        end
    end
    if wpos>1
        vect=ptable(1:(wpos-1),:); % only return usable portions of the data
    else
        vect=[]; % otherwise return blank table
    end
end

% calculates the least square linear regression of x and y
function [m, y_int, rsqr]=linreg(x, y)
    sum_x=sum(x);      % calc general sum_s 
    sum_y=sum(y);
    sum_xy=sum(x.*y);
    sum_xx=sum(x.*x);

    n=length(x);       % calc averages
    avgx=mean(x);
    avgy=mean(y);

    m=(n*sum_xy-sum_x*sum_y)/(n*sum_xx-sum_x.^2); % calc the m
    y_int = avgy - m .* avgx;                   % calc the y intercept
    sum_yres=sum((y - y_int - (m * x)).^2);     % calculate r^2
    sum_res=sum( (y-avgy).^2 );
    rsqr = (sum_res - sum_yres)/sum_res;
end

% forward merges segments that are similar
function bins_ = fwdmerge(dat, incr_by, nochg, maxdist)
    ismerged = false;
    i=1;
    link_=[];
    wpos=1;
    bins_ = zeros(size(dat,1),1);
    n_bins_max = max(dat(:,3));
    while ~(ismerged) % 
        if isempty(link_)
            link_ = find(dat(:,3)==i); % get first set
            [~,~,r2_] = linreg(dat(link_,1), dat(link_,2)); % and its r^2 value
        end
        if i < n_bins_max % not at end of list yet
            link_n =[];
            while (isempty(link_n) && i < n_bins_max)
                i=i+1;
                link_n = find(dat(:,3)==i);
            end
            [~,~,r2_n] = linreg(dat( ( [link_; link_n]) , 1), dat( ( [link_; link_n]) ,2)); 
            if ( (r2_n >= (r2_ + incr_by)) || (abs(r2_n-r2_) <= nochg) ) && ((( ( dat(link_n(1),1) - dat(link_(end),1) )^2 + ( dat(link_n(1),2) - dat(link_(end),2) )^2 )^.5) <= maxdist) % manual pdist calc
                link_ = [link_; link_n]; % merge those bins
                r2_ = r2_n;
            else % end of merged section
                bins_(link_) = wpos; % write bin number to output matrix
                wpos=wpos+1; % increment merged bin counter
                link_=[]; % clear out current link-up
            end
        end

        if i >= n_bins_max && ~isempty(link_)
            bins_(link_) = wpos; % write bin number to output matrix
            ismerged = true;
        end
    end
end

% scrubs a table of data of bad values
function table=lsscrub(data, minlen)
    [m, n] = size(data); % get table size
    ptable=zeros(m,n); % blank chopping table
    wpos=1; % initial write position
    for i=1:m
        if ( (data(i,5)-data(i,4)) < (minlen) ) % does the block have enough points?
            % do nothing
        else
            ptable(wpos,:)=data(i,:); % write good values to ptable
            wpos=wpos+1;
        end
    end
    table=ptable(1:(wpos-1),:); % only return usable portions of the data
end

% calculates the weighted euclidian distance between each adjacent pair of values in data [m x 2]
% returns vector list [(m-1) x 1]
function [list_] = mdist(data, xwgt, ywgt)
    [m,~] = size(data);
    if ~isempty(data)
        list_ = zeros(m,1);
        list_(1:(end-1)) = ( xwgt.*( data(2:end,1) - data(1:(end-1),1) ).^2 + ywgt.*( data(2:end,2) - data(1:(end-1),2) ).^2 ).^.5;
    else
        list_=[];
    end
end

% reimplementation of the pdist2 function (only returns lower triangle)
% stats lib free :P
function dat = pdist2_sq(data)
    [m,~]=size(data);
    dat = zeros(m,m);
    for i=1:m
        for j=1:m
            if j==i
                dat(i,j)=0;
            elseif j<i
                dat(i,j) = ( ( data(j,1) - data(i,1) ).^2 + ( data(j,2) - data(i,2) ).^2 ).^.5;
            else % should be a copy but we'ere only using the lower triangle
                dat(i,j)=0;
            end
        end
    end
end