function slamsim_withFastSLAM(Ts, vel_r, vel_l, SD, initd)
    %% FastSLAM Simulator
    % simulator for fastslam algorithms
    
    % sample setup for differential drive robot
    %
    % slamsim.m v8.1
    % Alexander T. Hypes 11/2013
    % Jared Strader 11/2013
    % Jarius Morton 9/2013
    
    % FastSLAM algorithm originally based on the version
    % created by Tim Bailey and Juan Nieto in 2004 of FastSLAM 1.0
    % http://www-personal.acfr.usyd.edu.au/tbailey/
    
    % Ts - time since last function call
    % vel_r - right wheel velocity
    % vel_l - left wheel velocity
    % SD - data structure with IMU data
    % initd - initialization boolean
    
    global FS                   % grab the global data var
    if initd
        %% Initialize Map
        env = 3;                % simulation map to show (only cosmetic)
        scale = 1;              % scale size of the map

        FS.mapsz = 10;          % mapsz x mapsz dimensions (meters)
        FS.mapres = 10;         % grid divisions per meter
        
        build_map(env, scale);  % build and plot map;
        marker_maker();         % generate polar coords for robot marker

        %% FastSLAM Initialization
        FS.map = zeros(FS.mapsz*FS.mapres); % generate empty map area
        
        % Variance
        FS.var_mv = [0.05;0.25];        % speed,rotation
        FS.var_ms = [0.1;0.5;0.0001];   % distance, angle, landmark id
        FS.R = [FS.var_ms(1),0,0;0,FS.var_ms(2),0;0,0,FS.var_ms(3)]; % covariance matrix

        % Data Association / Landmark Constants
        FS.min_lm_dist=0.3; % min distance between landmarks to be considered the same
        FS.max_lm_dist=0.7; % max distance between landmarks to be considered the same
        FS.max_lm = 50;     % max number of landmarks
        FS.num_part = 40;   % number of particles

        %% Simulation
        FS.data=zeros(2, 4); % build data table >> format [time xpos ypos heading]
        FS.data(1,:) = [0, 0, 0, pi/2]; % init starting orientation

        % Simulation Initialization
        FS.pos = FS.data(1,2:4)';   % init slam position
        FS.landmarks=[];            % init landmark table
        FS.data_table=[];           % init data table
        fov_width=620;  % scan lines to consider (58 deg FOV (Horizontal))
        dpl=58/640;     % degrees per horizontal scan line
        
        % produces translation table for x,y, pos from depth data
        FS.dtable = tand(((1:fov_width)-(fov_width/2)).*dpl)'; 

        % Initialize particles
        FS.particles=initParticles(FS.num_part,FS.max_lm,FS.pos);    

        % Update Plot + Save Handles
        [FS.h1, FS.h8] = plot_robot(FS.data(1,2),  FS.data(1,3),  FS.data(1,4), [], []);    % plot bot marker
        [FS.h2, ~] = ls_fov_plot(FS.data(1,2),  FS.data(1,3),  FS.data(1,4), []);           % plot/get kinect fov coords
        FS.h3 = lm_plot([],[]); % plot ones that are in view
        FS.h4 = [];
        FS.h5 = [];
        FS.h6 = [];
        FS.h7 = [];
        FS.use_vel_model=false; % use built-in velocity model
        FS.i=1;
    else
        %% Main Loop
        lsdata = depth2point(double(SD.LaserScan)',FS.dtable);      % translate laserscan data from polar to cartesian coordinates
        [cnrs, laserdat] = CornerDetect_nodep(lsdata, 4, 1);    % grab corners from data (loc rel. to robot)
        
        if FS.use_vel_model
            % use dead reckoning velocity model for pose
            FS.data(FS.i+1,:) = VelModelUpdate(FS.data(FS.i,:), Ts, vel_r, vel_l); % record motion values
        else
            % use rate gyro and built in functions for pose
            FS.data(FS.i+1,1)=FS.data(FS.i,1) + Ts;     % update time
            FS.data(FS.i+1,4)=FS.data(FS.i,4)-SD.R*Ts;  % update heading
            FS.data(FS.i+1,2)=SD.X;                     % update global x location
            FS.data(FS.i+1,3)=SD.Y;                     % update global y location
        end
        
        if isempty(cnrs) % build corner plot data
            inrange=[];
        else
            inrange = zeros(size(cnrs,1),2); % blank table
            inrange2 = rel_2_pol(cnrs/1000); % convert corners to polar (convert mm to m)
            inrange(:,1) =  inrange2(:,1) .* cos( inrange2(:,2) + FS.data(FS.i,4) ) + FS.data(FS.i,2); % convert back to x/y with proper heading adjustment
            inrange(:,2) =  inrange2(:,1) .* sin( inrange2(:,2) + FS.data(FS.i,4) ) + FS.data(FS.i,3);
        end
        
        if ~isempty(FS.h4) delete(FS.h4); FS.h4=[]; end % remove previous plot elements gracefully
        if ~isempty(FS.h5) delete(FS.h5); FS.h5=[]; end
        if ~isempty(FS.h6) delete(FS.h6); FS.h6=[]; end

        [FS.h1, FS.h8] = plot_robot(FS.data(FS.i+1,2),  FS.data(FS.i+1,3),  FS.data(FS.i+1,4), FS.h1, FS.h8);   % plot bot marker
        [FS.h2, ~] = ls_fov_plot(FS.data(FS.i+1,2),  FS.data(FS.i+1,3),  FS.data(FS.i+1,4), FS.h2);             % plot/get kinect fov coords
        FS.h3 = lm_plot(inrange, FS.h3);                                % plot detected corners that are in view
        FS.h4 = hud_update(FS.data(FS.i+1,:), inrange, vel_r, vel_l);   % update hud
        FS.h7 = live_scatter(laserdat, FS.data(FS.i,:), FS.h7);         % plot live laserscan data

        % Slam Algorithm
        cmd_mov = [(vel_r+vel_l)/2, SD.R];    % convert inputs for velocity motion model
        FS.pos=FS.data(FS.i+1,2:4);           % current position 
        
        % Move particles
        for index_j = 1:FS.num_part
            [FS.particles(index_j).position] = sampleParticle(FS.particles(index_j).position, cmd_mov, FS.var_mv, Ts);
            FS.particles(index_j).position(3) = FS.pos(3);
        end

        % Delete landmarks that are too close together
        if ~isempty(FS.data_table)
            for ii=1:size(FS.data_table,1)
                if(size(FS.data_table,1)<ii); break; end;
                jj=1;
                while jj<=size(FS.data_table,1)
                    if(ii~=jj)
                        dist=sqrt((FS.data_table(ii,1)-FS.data_table(jj,1))^2+(FS.data_table(ii,2)-FS.data_table(jj,2))^2);
                    else
                        dist=100;
                    end
                    if(dist<FS.min_lm_dist)
                        FS.data_table;
                        FS.data_table(jj,:)=[];
                    else
                        jj=jj+1;
                    end
                end
            end
        end
        
        % Get Observations and Data Association
        if ~isempty(inrange)
            FS.landmarks=dataAssociation(inrange,FS.data_table,FS.max_lm_dist)';
            assert( (size(FS.landmarks,2)<= FS.max_lm), 'slamsim: Excceeded max (%i) number of landmarks.', FS.max_lm);
        end
        
        if ~isempty(FS.landmarks)
            %Take measurements for each landmark
            for index_i = 1:size(FS.landmarks,2) 
                landmark = FS.landmarks(:, index_i);                        %extract landmark data
                [z_real, G] = getMeasurements(FS.pos, landmark, FS.var_ms); %calculate location
                landmark_distance(index_i) = z_real(1);                     %save distance
                landmark_angle(index_i) = z_real(2);                        %save angle

                % EKF PARTICLES
                for index_j = 1:FS.num_part
                    if(FS.particles(index_j).landmarks(index_i).seen == false) %if new landmark initialize
                        %Add landmark position to particle
                        FS.particles(index_j).landmarks(index_i).pos = [FS.particles(index_j).position(1) + cos(landmark_angle(index_i))*landmark_distance(index_i);
                                                                        FS.particles(index_j).position(2) + sin(landmark_angle(index_i))*landmark_distance(index_i);
                                                                        0];
                        %Add covariance to particle
                        FS.particles(index_j).landmarks(index_i).E = inv(G) * FS.R * inv(G)';
                        
                        %Update status of landmark in particle to seen
                        FS.particles(index_j).landmarks(index_i).seen = true;
                    else
                        % Get reading for believed landmark position
                        [z_p, Gp] = getMeasurements(FS.particles(index_j).position, FS.particles(index_j).landmarks(index_i).pos, [0;0]);
                        residual = z_real - z_p;

                        % Calculate the Kalman gain
                        Q = G' * FS.particles(index_j).landmarks(index_i).E * G + FS.R;
                        K = FS.particles(index_j).landmarks(index_i).E * G * inv(Q);

                        % Predict a new landmark position
                        FS.particles(index_j).landmarks(index_i).pos = FS.particles(index_j).landmarks(index_i).pos + K*(residual);

                        % Update the covariance of this landmark
                        FS.particles(index_j).landmarks(index_i).E = (eye(size(K)) - K*G')*FS.particles(index_j).landmarks(index_i).E;

                        % Update the weight of the particle
                        FS.particles(index_j).w = FS.particles(index_j).w * norm(2*pi*Q).^(-1/2)*exp(-1/2*(residual)'*inv(Q)*(residual));
                    end
                end
            end
            
            % Resample all particles based on their weights
            FS.particles = resampleParticle(FS.particles);
           
            % Obtain best guess for landmarks (average)
            for index_p = 1:size(FS.landmarks,2)
                if(FS.particles(1).landmarks(index_p).seen)
                    data_sum =[0;0;0];
                    for index_l = 1:length(FS.particles)
                        data_sum = data_sum + FS.particles(index_l).landmarks(index_p).pos;
                    end
                    FS.data_table(index_p,1) = data_sum(1) / length(FS.particles);
                    FS.data_table(index_p,2) = data_sum(2) / length(FS.particles);
                end
            end

            % Obtain best guess for position and heading (average)
            data_sum =[0;0;0];
            for index_p = 1:length(FS.particles)
                data_sum = data_sum + FS.particles(index_p).position;
            end
            FS.pos(1) = data_sum(1) / length(FS.particles);
            FS.pos(2) = data_sum(2) / length(FS.particles);
            FS.pos(3) = data_sum(3) / length(FS.particles);
        end

        % Position Vector for Plotting
        for index_i= 1:FS.num_part
            partVect(index_i,1)=FS.particles(index_i).position(1);
            partVect(index_i,2)=FS.particles(index_i).position(2);
        end

        % Plot particle positions
        hold on
        FS.h5 = scatter(partVect(:,1), partVect(:,2),'cp');
        hold off

        % Plot best guess landmark positions
        hold on
        if ~isempty(FS.data_table)
            FS.h6 = scatter(FS.data_table(:,1),FS.data_table(:,2),'go');
        else
            FS.h6 = scatter(0,0,'go');
        end
        hold off
        
        % Pause for GUI
        pause(.00000001);
        
        FS.i=1;
        FS.data(1,:)=FS.data(2,:);
    end
end

%% Subfunctions

% converts detected landmarks into relative polar vectors
function h = live_scatter(laserdat, datain, h)
    global FS
    dataout=[];
    if ~isempty(laserdat)
        ld_polar = rel_2_pol(laserdat/1000);% convert live data to polar;
        dataout = zeros(size(ld_polar,1), 2); % build empty matrix v
        dataout(:,1) =  ld_polar(:,1) .* cos( ld_polar(:,2) + datain(4) ) + datain(2); % rotate polar angle by heading for both x and y data
        dataout(:,2) =  ld_polar(:,1) .* sin( ld_polar(:,2) + datain(4) ) + datain(3);
        
        % record scatter values for occupancy grid creation
        % (simply paints all cells that the laserscaner sees with a 1)
        ldata_r=dataout;
        ldata_r(:,1) = round((ldata_r(:,1)+FS.mapsz/2 ) * FS.mapres);
        ldata_r(:,2) = round((ldata_r(:,2)+FS.mapsz/2 ) * FS.mapres);
        for i=1:length(laserdat)
            FS.map(ldata_r(i,1),ldata_r(i,2))=1;
        end
    end
    hold on % plots laserscan data if necessary
    if ~isempty(dataout)
        if isempty(h) 
            h = scatter(dataout(:,1), dataout(:,2), 'm.');
        else
            delete(h)
            h = scatter(dataout(:,1), dataout(:,2), 'm.');
        end
    end
    hold off
end

% landmark plotter
function h = lm_plot(inrange, h)
    if isempty(h) % plots landmarks if they've never been plotted
        if ~isempty(inrange)
            hold on;
            h = plot(inrange(:,1), inrange(:,2),'rs'); % plot landmarks
            hold off
        end
    else % otherwise, only update the list of items to plot
        if ~isempty(inrange)
            set(h,'XData',inrange(:,1),'YData',inrange(:,2));
        else
            set(h,'XData',[],'YData',[]);
        end
    end
end

% converts xy data into polar form about 0,0
function dataout = rel_2_pol(datain)
    if ~isempty(datain)
        dataout = zeros(size(datain,1), 2); % build empty matrix v 
        dataout(:,1) = sqrt( datain(:,2).^2 + datain(:,1).^2 ); % line length (r)
        dataout(:,2) = mod( atan2( datain(:,2), datain(:,1)) - pi/2, 2*pi); % rel. polar angle of lms
    else
        dataout=[];
    end
end

% plots the heads up display
function h = hud_update(data, inrange, v_r, v_l)
    str1(1) = {strcat( 'Time: ',    num2str( data(1),           '%0.2f' ) )};
    str1(3) = {strcat( 'X:    ',    num2str( data(2),           '%0.2f' ) )};
    str1(4) = {strcat( 'Y:    ',    num2str( data(3),           '%0.2f' ) )};
    str1(2) = {strcat( '\theta:   ',num2str( mod( data(4)*180/pi , 360),    '%0.1f'), '\circ' )};
    str1(5) = {strcat( 'LMs:  ',    num2str( size(inrange,1),   '%i') )};

    str2(1) = {strcat( 'V_r: ',     num2str( v_r,               '%0.2f m/s' ) )};
    str2(2) = {strcat( 'V_l: ',     num2str( v_l,               '%0.2f m/s' ) )};
    data(1:2) = xlim();
    data(3:4) = ylim();
    data(5) = data(2)-data(1); % total x width
    data(6) = data(4)-data(3); % total y height
    h = [ text(data(2) - data(5)*.06, data(4) - data(6)*.06, str2,'FontSize',12,'FontWeight','Bold', 'VerticalAlignment', 'top', 'HorizontalAlignment', 'right')';
          text(data(5)*.06 + data(1), data(4) - data(6)*.06, str1,'FontSize',13,'FontWeight','Bold', 'VerticalAlignment', 'top')'];
end

% environmental map plotter
function h = plot_map(mapin, ax)
    if ~isempty(mapin) % dont plot empty maps
        hold on % allow multiple patch commands
        h=figure(1); % create new figure window
        r=2 * max(ax); % get max visible dimension
        patch([-r -r r r],[-r r r -r],[.5 .5 .5]); % plot grey outside (bigger than is visible)
        patch(mapin(:,1),mapin(:,2),[.2 .2 .2]); % plot actual map
        hold off
        axis(ax); % fix axis plotting area
        whitebg('k');
    end
    set(h, 'Position', [100 100 600 600])
end

% environmental map generator
function [lm, ax, mapcoords] = build_map(env, scale)
    if env == 0 
        lm = (     [0 0; 0 2; 3 2; 3 0]-.5) * scale; % landmark list
        mapcoords=([0 0; 0 2; 3 2; 3 0]-.5) * scale; % without doors
        ax=([-.25 3.25 -.25 3.25]-.5) * scale; % axis list
    elseif env==1 % simulated hallway
        lm = (     [0 0; 0 2.2; 1 2.2;           2 3; 2.5 3; 2.5 2; 3 2; 3 0] - .5) * scale; % landmark list
        mapcoords=([0 0; 0 2.2; 1 2.2; 1 8; 2 8; 2 3; 2.5 3; 2.5 2; 3 2; 3 0] - .5) * scale; % without doors
        ax=([-.25 3.25 -.25 3.25]-.5) * scale; % axis list
    elseif env == 2 % simulated hallway with doors
        lm = (     [0 0; 0 .25; -.05 .25; -.05 .75; 0 .75; 0 2.2; 1 2.2;           2 3; 2.5 3; 2.5 2.75; 2.55 2.75; 2.55 2.25; 2.5 2.25; 2.5 2; 3 2; 3 1.8; 3.05 1.8; 3.05 1.3; 3 1.3; 3 .75; 3.05 .75; 3.05 .25; 3 .25; 3 0] - .5) * scale; % landmark list
        mapcoords=([0 0; 0 .25; -.05 .25; -.05 .75; 0 .75; 0 2.2; 1 2.2; 1 8; 2 8; 2 3; 2.5 3; 2.5 2.75; 2.55 2.75; 2.55 2.25; 2.5 2.25; 2.5 2; 3 2; 3 1.8; 3.05 1.8; 3.05 1.3; 3 1.3; 3 .75; 3.05 .75; 3.05 .25; 3 .25; 3 0] - .5) * scale; % with doors
        ax=([-.25 3.25 -.25 3.25]-.5) * scale;
    elseif env == 3  % simple square
        lm = (     [-4 -4; -4 4; 4 4; 4 -4] ) * scale;
        mapcoords=([-4 -4; -4 4; 4 4; 4 -4] ) * scale; 
        ax=([-4.5 4.5 -4.5 4.5]) * scale;        
    end
    plot_map(mapcoords, ax); % plot the loaded map
    title('SLAM Simulator','FontSize',14,'FontWeight','Bold');
end

% landmark LOS/inrange check (only works for the environment landmarks)
function inrange = lm_rangecheck(lm, rf_area, xpos, ypos, mapcoords)
    % first check that lm is within the kincts fov
    list_ = inpolygon( lm(:,1), lm(:,2), rf_area(:,1), rf_area(:,2) );
    inrange = lm( list_ , : );
    
    % then check for line of sight
    ppi=.05; % 1/20 % los check resolution
    inrangelos = zeros(size(inrange,1), 2); % init LOS check table
    wout=0; % write pos counter
    for i=1:size(inrange,1)
        deltax=( inrange(i,1)-xpos ); % calculate deltas
        deltay=( inrange(i,2)-ypos );
        llen = 1 / sqrt( deltax^2 + deltay^2 ); % find line lengths
        
        xtab=[xpos:(ppi*llen*deltax):(xpos+deltax)]'; % build line vectors
        ytab=[ypos:(ppi*llen*deltay):(ypos+deltay)]'; 

        % check for los by drawing line from lm to robot and checking if
        % its out of the map anywhere
        if sum( inpolygon( xtab, ytab, mapcoords(:,1), mapcoords(:,2) ) ) >= length(xtab)
            wout=wout+1; % record good values
            inrangelos(wout,:) = inrange(i,:);
        end
    end
    inrange = inrangelos(1:wout,:); % only return useful data
end

% discretized velocity model (dead reckoning only)
% datain/dataout format: [time xpos ypos heading]
function [dataout] = VelModelUpdate(datain, Ts, vel_r, vel_l)
    Vs = (vel_r + vel_l ) / 2; % true velocity
    omega = (vel_r - vel_l ) / .263; % 0.263 is the wheel track (in meters)
    dataout=[ (datain(1)+Ts), 0, 0, 0];
    dataout(4) = datain(4) + omega*Ts; % get theta from last update / no change
    dataout(2) = datain(2) + Ts * Vs * cos( dataout(4) ); % new xpos
    dataout(3) = datain(3) + Ts * Vs * sin( dataout(4) ); % new ypos
end

% generates the x/y tables for the robot marker
function marker_maker()
    global FS
    r=.35; % polar radius (O.D. of robot)
    theta=[pi/2:pi/12:pi*5/2]; % full circle
    FS.cir = [ r*cos(theta)' r*sin(theta)']; % circle points
    % arrow points
    FS.arr = [pi/2 pi*4/3 pi*3/2 pi*5/3 pi/2]' - pi/2;
end

% plots the robot as a circle with an arrow
function [h, h2] = plot_robot(xpos, ypos, heading, h, h2)
    global FS
    r=.35;  % polar radius (O.D. of robot)
    n_trail = 150; % only keep the last n_trail points
    arr__ = [ r*cos((FS.arr + heading)) r*sin(FS.arr+ heading)];
    arr__(3,:)=arr__(3,:)*2/3;
    hold on
    if isempty(h)
        h = [ fill( FS.cir(:,1)+xpos , FS.cir(:,2)+ypos , [.3 .71 .98]);
            fill(arr__(:,1)+xpos , arr__(:,2)+ypos, [1 0 0]) ];
        set(h,'facealpha',.3)
        set(h,'edgealpha',.5)
    else
        set(h(1),'XData',FS.cir(:,1)+xpos,'YData',FS.cir(:,2)+ypos)
        set(h(2),'XData',arr__(:,1)+xpos, 'YData', arr__(:,2)+ypos)    
    end
    if isempty(h2) % init if no table given
        h2=zeros(n_trail+2,2);
        h2(end-1,1)=1; % set write bit
        h2(1,1:2) = [xpos,ypos];
        h2(end,1) = plot(h2(1:h2(end-1,1),1), h2(1:h2(end-1,1),2),'b.');
    else % if tables already there
        h2(end-1,1)=h2(end-1,1) + 1; % increase the wbit by 1
        if h2(end-1,1)>n_trail % rollover
            h2(end-1,1) = 1;
            h2(end-1,2) = 2; % set magic bit
        end
        h2(h2(end-1,1),1:2) = [xpos,ypos]; % record x/y position
    end
    if (h2(end-1,2)) % if the magic bit is set
        set(h2(end,1),'XData',h2(1:(end-2),1),'YData',h2(1:(end-2),2)) % update plot with everything
    else %otherwise only show first few values
        set(h2(end,1),'XData',h2(1:h2(end-1,1),1),'YData',h2(1:h2(end-1,1),2))
    end
    hold off
end

% fov rangefinder plotter
function [h, rf_area] = ls_fov_plot(xpos, ypos, heading, h)
    fang = 1.01229096615671; % horizontal fov (deg)   fang = 58 * pi/180;
    dmax = 4.70467331848513; % max sensing distance   dmax = ( 162 * .0254 ) / cos(fang/2);
    dmin = 0.92931818636743; % min sensing distance   dmin = (  32 * .0254 ) / cos(fang/2);
    
    afang = mod( ([ fang/2; -fang/2] + heading), 2* pi); % abs angles
    rf_area = zeros(4,2);
    rf_area(:,1) = [dmin*cos(afang(2)); dmin*cos(afang(1)); dmax*cos(afang(1)); dmax*cos(afang(2))] + xpos; 
    rf_area(:,2) = [dmin*sin(afang(2)); dmin*sin(afang(1)); dmax*sin(afang(1)); dmax*sin(afang(2))] + ypos;    
    hold on
    if isempty(h)
        h = [patch( rf_area(:,1), rf_area(:,2) , [1 0 0]); patch( [ xpos; rf_area(1:2,1)] , [ypos; rf_area(1:2,2)], [1 1 1])];
        set(h,'facealpha',.05)
        set(h,'edgealpha',.1)    
    else
        set(h(1),'XData',rf_area(:,1),'YData',rf_area(:,2))
        set(h(2),'XData',[ xpos; rf_area(1:2,1)], 'YData', [ypos; rf_area(1:2,2)])
    end
    hold off
end

% converts depth data from a single point into x/y
function table = depth2point(data,dtable)
    table=[dtable.*data data];
end